-- automatic differentiation, one variable

local ad={}

local function ad_new(a0,a1)
	local a={}
	a[0]=a0
	a[1]=a1
	return setmetatable(a,ad)
end

function ad.__tostring(t)
	return string.format("[%g,%g]",t[0],t[1])
end

function ad.var(a0)
	return ad_new(a0,1)
end

function ad.__unm(b)
	local a=ad_new()
	a[0]=-b[0]
	a[1]=-b[1]
	return a
end

function ad.__add(b,c)
	local a=ad_new()
	if type(b)=="number" then
		a[0]=b+c[0]
		a[1]=  c[1]
	elseif type(c)=="number" then
		a[0]=b[0]+c
		a[1]=b[1]
	else
		a[0]=b[0]+c[0]
		a[1]=b[1]+c[1]
	end
	return a
end

function ad.__sub(b,c)
	local a=ad_new()
	if type(b)=="number" then
		a[0]=b-c[0]
		a[1]= -c[1]
	elseif type(c)=="number" then
		a[0]=b[0]-c
		a[1]=b[1]
	else
		a[0]=b[0]-c[0]
		a[1]=b[1]-c[1]
	end
	return a
end

function ad.__mul(b,c)
	local a=ad_new()
	if type(b)=="number" then
		a[0]=b*c[0]
		a[1]=b*c[1]
	elseif type(c)=="number" then
		a[0]=c*b[0]
		a[1]=c*b[1]
	else
		a[0]=b[0]*c[0]
		a[1]=b[0]*c[1]+b[1]*c[0]
	end
	return a
end

function ad.__pow(b,n)
	assert(n>0 and n==math.floor(n))
	local a=ad_new()
	a[0]=b[0]^n
	a[1]=n*b[0]^(n-1)*b[1]
	return a
end

ad.new=ad_new

return ad
