local ia=require"ia"
local interval=ia.interval
local range=ia.range
local plotboxes=true
local plotgraph=true
local plotpoints=true

local function curve(t)
	local a=1
	local x=a*2*(t^2-3)
	local y=a*t*(t^2-3)
	return x,y
end

local function coords(xmin,xmax,ymin,ymax)
	local W,H=love.graphics.getDimensions()
	love.graphics.translate(0,H)
	love.graphics.scale(1,-1)
	local dx=xmax-xmin
	local dy=ymax-ymin
	local s=math.min(W/dx,H/dy)
	love.graphics.translate((W-s*dx)/2,(H-s*dy)/2)
	love.graphics.scale(s,s)
	love.graphics.translate(-xmin,-ymin)
	love.graphics.setLineWidth(0)
end

local function graph(xmin,xmax,ymin,ymax,tmin,tmax,n)
	love.graphics.setColor(100,100,100)
	love.graphics.setLineWidth(0.002*(xmax-xmin))
	local dt=(tmax-tmin)/n
	local x1,y1=curve(tmin)
	local x2,y2
	for t=tmin,tmax,dt do
		x2,y2=curve(t)
		love.graphics.line(x1,y1,x2,y2)
		x1,y1=x2,y2
	end
	x2,y2=curve(tmax)
	love.graphics.line(x1,y1,x2,y2)
end

local function box(tmin,tmax)
	local t=interval(tmin,tmax)
	local x,y=curve(t)
	local xmin,xmax=range(x)
	local ymin,ymax=range(y)
	love.graphics.setColor(200,200,255)
	love.graphics.rectangle("fill",xmin,ymin,xmax-xmin,ymax-ymin)
	love.graphics.setColor(0,0,255)
	love.graphics.rectangle("line",xmin,ymin,xmax-xmin,ymax-ymin)
end

local function boxes(tmin,tmax,n)
	local dt=(tmax-tmin)/n
	for t=tmin,tmax-dt,dt do
		box(t,t+dt)
	end
	box(tmax-dt,tmax)
end

local function point(t)
	local x,y=curve(t)
	love.graphics.points(x,y)
end

local function points(tmin,tmax,n)
	love.graphics.setColor(100,100,100)
	local dt=(tmax-tmin)/n
	for t=tmin,tmax,dt do
		point(t)
	end
	point(tmax)
end

local function plot(xmin,xmax,ymin,ymax,tmin,tmax,n)
	coords(xmin,xmax,ymin,ymax)
	if plotboxes then
		boxes(tmin,tmax,n)
	end
	if plotgraph then
		graph(xmin,xmax,ymin,ymax,tmin,tmax,500)
	end
	if plotpoints then
		points(tmin,tmax,n)
	end
end

function love.load()
	--love.window.setFullscreen(true)
	love.window.setMode(640,480,{resizable=true})
	love.window.setPosition(0,0)
	love.window.setTitle("curve plotting with interval arithmetic")
	love.graphics.setBackgroundColor(255,255,255)
	love.graphics.clear()
	love.graphics.setFont(love.graphics.newFont(96))
	love.keyboard.setKeyRepeat(true)
	love.graphics.setPointSize(8)
end

local N=0

function love.keypressed(key)
	if key == 'q' or key == 'escape' then
		love.event.quit()
	elseif key == 's' then
		love.graphics.newScreenshot():encode('png','output.png')
	elseif key == '[' then
		N=N-1
		if N<0 then N=0 end
	elseif key == ']' then
		N=N+1
		if N>15 then N=15 end
	elseif key == 'b' then
		plotboxes= not plotboxes
	elseif key == 'g' then
		plotgraph= not plotgraph
	elseif key == 'p' then
		plotpoints= not plotpoints
	end
end

function love.draw()
	plot(-7,5,-4,4,-2,2.2,2^N)
end
