local ia=require"ia"
local interval=ia.interval
local range=ia.range
local plotgraph=true
local plotbox={true,true,true}
local g={
	function(x) return x^3-x		end,
	function(x) return x*(x^2-1)		end,
	function(x) return x*(x-1)*(x+1)	end,
}
local T=0.5

local f=g[1]

local function coords(xmin,xmax,ymin,ymax)
	local W,H=love.graphics.getDimensions()
	love.graphics.translate(0,H)
	love.graphics.scale(1,-1)
	local dx=xmax-xmin
	local dy=ymax-ymin
	local s=math.min(W/dx,H/dy)
	love.graphics.translate((W-s*dx)/2,(H-s*dy)/2)
	love.graphics.scale(s,s)
	love.graphics.translate(-xmin,-ymin)
	love.graphics.setLineWidth(0)
end

local function axes(xmin,xmax,ymin,ymax)
	love.graphics.setColor(200,200,200)
	love.graphics.setLineWidth(0)
	love.graphics.line(xmin,0,xmax,0)
end

local function graph(xmin,xmax,ymin,ymax,n)
	love.graphics.setColor(100,100,100)
	love.graphics.setLineWidth(0.002*(xmax-xmin))
	local dx=(xmax-xmin)/n
	local x1,y1=xmin,f(xmin)
	local x2,y2
	for x=xmin,xmax,dx do
		x2,y2=x,f(x)
		love.graphics.line(x1,y1,x2,y2)
		x1,y1=x2,y2
	end
	x2,y2=xmax,f(xmax)
	love.graphics.line(x1,y1,x2,y2)
end

local function box(x,y,i)
	local xmin,xmax=range(x)
	local ymin,ymax=range(y)
	if i==1 then
		love.graphics.setColor(255,200,200,200)
		love.graphics.rectangle("fill",xmin,ymin,xmax-xmin,ymax-ymin)
		love.graphics.setColor(255,0,0)
	elseif i==2 then
		love.graphics.setColor(200,255,200,200)
		love.graphics.rectangle("fill",xmin,ymin,xmax-xmin,ymax-ymin)
		love.graphics.setColor(0,0,255)
	elseif i==3 then
		love.graphics.setColor(200,200,255,200)
		love.graphics.rectangle("fill",xmin,ymin,xmax-xmin,ymax-ymin)
		love.graphics.setColor(0,0,255)
	end
	love.graphics.rectangle("line",xmin,ymin,xmax-xmin,ymax-ymin)
end

local function visit(xmin,xmax,n)
	local xmid=xmin+T*(xmax-xmin)
	local dx=(xmax-xmin)/n/2
	local x=interval(xmid-dx,xmid+dx)
	local j,dmin=0,1/0 -- inf
	for i=1,3 do
		local y=g[i](x)
		local ymin,ymax=range(y)
		local d=ymax-ymin
		if d<dmin then
			j,dmin=i,d
		end
		box(x,g[i](x),i)
	end
	box(x,g[j](x),j)
	return j
end

local function result(n,j)
	love.graphics.origin()
	love.graphics.setColor(100,100,100)
	love.graphics.print(string.format("%d\t%d",n,j),10,10)
end

local function plot(xmin,xmax,ymin,ymax,n)
	coords(xmin,xmax,ymin,ymax)
	axes(xmin,xmax,ymin,ymax)
	local j=visit(xmin,xmax,n)
	if plotgraph then
		graph(xmin,xmax,ymin,ymax,128)
	end
	result(n,j)
end

function love.load()
	--love.window.setFullscreen(true)
	love.window.setMode(640,480,{resizable=true})
	love.window.setPosition(0,0)
	love.window.setTitle("quality of interval estimates")
	love.graphics.setBackgroundColor(255,255,255)
	love.graphics.clear()
	love.graphics.setFont(love.graphics.newFont(96))
	love.keyboard.setKeyRepeat(true)
end

local N=0

function love.keypressed(key)
	local dt=2^-6
	if key == 'q' or key == 'escape' then
		love.event.quit()
	elseif key == 's' then
		love.graphics.newScreenshot():encode('png','output.png')
	elseif key == '[' then
		N=N-1
		if N<0 then N=0 end
	elseif key == ']' then
		N=N+1
		if N>15 then N=15 end
	elseif key == '1' then
		plotbox[1]= not plotbox[1]
	elseif key == '2' then
		plotbox[2]= not plotbox[2]
	elseif key == '3' then
		plotbox[3]= not plotbox[3]
	elseif key == 'g' then
		plotgraph= not plotgraph
	elseif key == 'left' then
		T=T-dt
	elseif key == 'right' then
		T=T+dt
	end
end

function love.draw()
	local xmin,xmax=-2,2
	local ymin,ymax=-2,2
	local xmin,xmax=-4,4
	local ymin,ymax=-4,4
	plot(xmin,xmax,ymin,ymax,2^N)
end
