local N=0
local M=3

local function coords(xmin,xmax,ymin,ymax)
	local W,H=love.graphics.getDimensions()
	love.graphics.translate(0,H)
	love.graphics.scale(1,-1)
	local dx=xmax-xmin
	local dy=ymax-ymin
	local s=math.min(W/dx,H/dy)
	love.graphics.translate((W-s*dx)/2,(H-s*dy)/2)
	love.graphics.scale(s,s)
	love.graphics.translate(-xmin,-ymin)
	love.graphics.setLineWidth(0)
end

local function axes(xmin,xmax,ymin,ymax)
	love.graphics.setColor(100,100,100)
	love.graphics.line(xmin,0,xmax,0)
	love.graphics.setColor(255,100,100)
	for x=xmin+1,xmax-1 do
		love.graphics.line(x,ymin,x,ymax)
	end
end

local function interval(n,s)
	n=2^n
	local u=1/n
	for i=0,n-1 do
		local x=s*(1+i*u)
		local y=1
		if s<0 then x=-1/x end
		love.graphics.line(x,-y,x,y)
	end
end

local function plot(xmin,xmax,ymin,ymax)
	coords(xmin,xmax,ymin,ymax)
	love.graphics.setColor(0,0,255)
	love.graphics.setLineWidth(0.02)
	--interval(N,-1)
	interval(N,1)
	interval(N,2)
	interval(N,4)
	axes(xmin,xmax,ymin,ymax)
	love.graphics.origin()
	love.graphics.setColor(100,100,100)
	love.graphics.print(string.format("%d",N),10,10)
end

function love.load()
	--love.window.setFullscreen(true)
	love.window.setMode(1000,300,{resizable=true})
	love.window.setPosition(0,0)
	love.window.setTitle("floating-point numbers")
	love.graphics.setBackgroundColor(255,255,255)
	love.graphics.clear()
	love.graphics.setFont(love.graphics.newFont(96))
	love.keyboard.setKeyRepeat(true)
end

function love.keypressed(key)
	if key == 'q' or key == 'escape' then
		love.event.quit()
	elseif key == 's' then
		love.graphics.newScreenshot():encode('png','output.png')
	elseif key == '[' then
		N=N-1
		if N<0 then N=0 end
	elseif key == ']' then
		N=N+1
	end
end

function love.draw()
	plot(-1,2^M+1,-1,1)
end
