local ia=require"ia"
local interval=ia.interval
local range=ia.range

local function f(x)
	--return x^3-x-1
	--return x*(x*(x*(x-2)-1.44)+2.88)
	--return (x+1.2)*(x-1.2)*(x-2.5)*x
	--x=-x+(-1.125+2.125)
	return (x+1.2)*(x-1.2)*(x-2)*x
end

local function coords(xmin,xmax,ymin,ymax)
	local W,H=love.graphics.getDimensions()
	love.graphics.translate(0,H)
	love.graphics.scale(1,-1)
	local dx=xmax-xmin
	local dy=ymax-ymin
	local s=math.min(W/dx,H/dy)
	love.graphics.translate((W-s*dx)/2,(H-s*dy)/2)
	love.graphics.scale(s,s)
	love.graphics.translate(-xmin,-ymin)
	love.graphics.setLineWidth(0)
end

local function axes(xmin,xmax,ymin,ymax)
	love.graphics.setColor(200,200,200)
	love.graphics.setLineWidth(0)
	love.graphics.line(xmin,0,xmax,0)
end

local function graph(xmin,xmax,ymin,ymax,n)
	love.graphics.setColor(100,100,100)
	love.graphics.setLineWidth(0.002*(xmax-xmin))
	local dx=(xmax-xmin)/n
	local x1,y1=xmin,f(xmin)
	local x2,y2
	for x=xmin,xmax,dx do
		x2,y2=x,f(x)
		love.graphics.line(x1,y1,x2,y2)
		x1,y1=x2,y2
	end
	x2,y2=xmax,f(xmax)
	love.graphics.line(x1,y1,x2,y2)
end

local function box(xmin,xmax,ymin,ymax)
	love.graphics.setColor(200,200,255)
	love.graphics.rectangle("fill",xmin,ymin,xmax-xmin,ymax-ymin)
	love.graphics.setColor(0,0,255)
	love.graphics.rectangle("line",xmin,ymin,xmax-xmin,ymax-ymin)
end

local function optimize(xmin,xmax,ymin,ymax,n)
	local inf=1/0
	local Ymin,Ymax=inf,-inf
	love.graphics.setLineWidth(0)
	local dx=(xmax-xmin)/n
	for x=xmin,xmax-dx,dx do
		local x=interval(x,x+dx)
		local y=f(x)
		local xmin,xmax=range(x)
		local ymin,ymax=range(y)
		box(xmin,xmax,ymin,ymax)
		if ymin<Ymin then
			love.graphics.setColor(255,0,0)
			love.graphics.line(xmin,ymin,xmax,ymin)
		end
		if ymax>Ymax then
			love.graphics.setColor(0,0,255)
			love.graphics.line(xmin,ymax,xmax,ymax)
		end
		Ymin=math.min(Ymin,ymin)
		Ymax=math.max(Ymax,ymax)
	end
	return Ymin,Ymax
end

local function result(xmin,xmax,Ymin,Ymax,n)
	love.graphics.setColor(255,0,0)
	love.graphics.setLineWidth(0.005*(xmax-xmin))
	love.graphics.line(xmin,Ymin,xmax,Ymin)
	love.graphics.setColor(0,0,255)
	love.graphics.line(xmin,Ymax,xmax,Ymax)
	love.graphics.origin()
	love.graphics.setColor(100,100,100)
	love.graphics.print(string.format("%d\t%.4f\t%.4f",n,Ymin,Ymax),10,10)
end

local function plot(xmin,xmax,ymin,ymax,n)
	coords(xmin,xmax,ymin,ymax)
	axes(xmin,xmax,ymin,ymax)
	local Ymin,Ymax=optimize(xmin,xmax,ymin,ymax,2^n)
	graph(xmin,xmax,ymin,ymax,128)
	result(xmin,xmax,Ymin,Ymax,n)
end

function love.load()
	--love.window.setFullscreen(true)
	love.window.setMode(640,480,{resizable=true})
	love.window.setPosition(0,0)
	love.window.setTitle("global optimization with interval arithmetic - uniform")
	love.graphics.setBackgroundColor(255,255,255)
	love.graphics.clear()
	love.graphics.setFont(love.graphics.newFont(96))
	love.keyboard.setKeyRepeat(true)
end

local N=0

function love.keypressed(key)
	if key == 'q' or key == 'escape' then
		love.event.quit()
	elseif key == 's' then
		love.graphics.newScreenshot():encode('png','output.png')
	elseif key == '[' then
		N=N-1
		if N<0 then N=0 end
	elseif key == ']' then
		N=N+1
		if N>15 then N=15 end
	end
end

function love.draw()
	local xmin,xmax=-1.125,2.125
	local ymin,ymax=-2.3,1.5
	plot(xmin,xmax,ymin,ymax,N)
end
