local aa=require"aa1"
local interval=aa.interval
local range=aa.range
local plotbox=true
local plotcolors=false
local plotgraph=true
local plotzonotope=false

local function f(x)
	return (x+1.2)*(x-1.2)*(x-2)*x
	--return x^4-2*x^3-1.44*x^2+2.88*x
	--return x*(x*(x*(x-2.)-1.44)+2.88)
end

local function coords(xmin,xmax,ymin,ymax)
	local W,H=love.graphics.getDimensions()
	love.graphics.translate(0,H)
	love.graphics.scale(1,-1)
	local dx=xmax-xmin
	local dy=ymax-ymin
	local s=math.min(W/dx,H/dy)
	love.graphics.translate((W-s*dx)/2,(H-s*dy)/2)
	love.graphics.scale(s,s)
	love.graphics.translate(-xmin,-ymin)
	love.graphics.setLineWidth(0)
end

local function axes(xmin,xmax,ymin,ymax)
	love.graphics.setColor(200,200,200)
	love.graphics.setLineWidth(0)
	love.graphics.line(xmin,0,xmax,0)
end

local function graph(xmin,xmax,ymin,ymax,n)
	love.graphics.setColor(100,100,100)
	love.graphics.setLineWidth(0.002*(xmax-xmin))
	local dx=(xmax-xmin)/n
	local x1,y1=xmin,f(xmin)
	local x2,y2
	for x=xmin,xmax,dx do
		x2,y2=x,f(x)
		love.graphics.line(x1,y1,x2,y2)
		x1,y1=x2,y2
	end
	x2,y2=xmax,f(xmax)
	love.graphics.line(x1,y1,x2,y2)
end

local function box(xmin,xmax,ymin,ymax)
	local r,g,b
	local s=0.8*255
	if ymin>0 then
		r,g,b=255,0,0
	elseif ymax<0 then
		r,g,b=0,0,255
	else
		r,g,b=0,255,0
	end
	if not plotcolors then
		r,g,b=0,0,255
	end
	love.graphics.setLineWidth(0)
	love.graphics.setColor(s+r,s+g,s+b)
	love.graphics.rectangle("fill",xmin,ymin,xmax-xmin,ymax-ymin)
	love.graphics.setColor(r,g,b)
	love.graphics.rectangle("line",xmin,ymin,xmax-xmin,ymax-ymin)
end

local function zonotope(x,y)
	local xmin,xmax=range(x)
	local x1=xmin
	local y1=y[0]-y[1]-y[2]
	local x2=xmin
	local y2=y[0]-y[1]+y[2]
	local x3=xmax
	local y3=y[0]+y[1]+y[2]
	local x4=xmax
	local y4=y[0]+y[1]-y[2]
	love.graphics.setColor(255,200,200)
	love.graphics.polygon("fill",x1,y1,x2,y2,x3,y3,x4,y4)
	love.graphics.setColor(255,0,0)
	love.graphics.polygon("line",x1,y1,x2,y2,x3,y3,x4,y4)
end

local function boxes(xmin,xmax,ymin,ymax,n)
	local dx=(xmax-xmin)/n
	for x=xmin,xmax-dx,dx do
		local x=interval(x,x+dx)
		local y=f(x)
		local xmin,xmax=range(x)
		local ymin,ymax=range(y)
		if plotbox then
			box(xmin,xmax,ymin,ymax)
		end
		if plotzonotope then
			zonotope(x,y)
	end
		end
end

local function result(xmin,xmax,n)
	love.graphics.origin()
	love.graphics.setColor(100,100,100)
	love.graphics.print(string.format("%d",n),10,10)
end

local function plot(xmin,xmax,ymin,ymax,n)
	coords(xmin,xmax,ymin,ymax)
	axes(xmin,xmax,ymin,ymax)
	boxes(xmin,xmax,ymin,ymax,2^n)
	graph(xmin,xmax,ymin,ymax,128)
	result(xmin,xmax,n)
end

function love.load()
	--love.window.setFullscreen(true)
	love.window.setMode(640,480,{resizable=true})
	love.window.setMode(1024,1024,{resizable=true})
	love.window.setPosition(0,0)
	love.window.setTitle("graph plotting with affine arithmetic")
	love.graphics.setBackgroundColor(255,255,255)
	love.graphics.clear()
	love.graphics.setFont(love.graphics.newFont(96))
	love.keyboard.setKeyRepeat(true)
end

local N=0

function love.keypressed(key)
	if key == 'q' or key == 'escape' then
		love.event.quit()
	elseif key == 's' then
		love.graphics.newScreenshot():encode('png','output.png')
	elseif key == '[' then
		N=N-1
		if N<0 then N=0 end
	elseif key == ']' then
		N=N+1
		if N>15 then N=15 end
	elseif key == 'b' then
		plotbox= not plotbox
	elseif key == 'c' then
		plotcolors= not plotcolors
	elseif key == 'z' then
		plotzonotope= not plotzonotope
	end
end

function love.draw()
	local L=3
	local xmin,xmax=-L,L
	local ymin,ymax=-L,L
	plot(xmin,xmax,ymin,ymax,N)
end
