local plotgraph=true
local plotpoints=true
local plotuniform=true
local plotcolors=false
local seed=2018
local Ymin=-1.25

local function random(a,b)
	return a+(b-a)*math.random()
end

local function curve(t)
	local x=t
	local y=math.cos(t)*math.sin(t^2)
	return x,y
end

local function coords(xmin,xmax,ymin,ymax)
	local W,H=love.graphics.getDimensions()
	love.graphics.translate(0,H)
	love.graphics.scale(1,-1)
	local dx=xmax-xmin
	local dy=ymax-ymin
	local s=math.min(W/dx,H/dy)
	love.graphics.translate((W-s*dx)/2,(H-s*dy)/2)
	love.graphics.scale(s,s)
	love.graphics.translate(-xmin,-ymin)
	love.graphics.setLineWidth(0)
end

local function axes(xmin,xmax,ymin,ymax)
	love.graphics.setColor(200,200,200)
	love.graphics.setLineWidth(0)
	love.graphics.line(xmin,0,xmax,0)
	love.graphics.line(0,ymin,0,ymax)
end

local function graph(xmin,xmax,ymin,ymax,tmin,tmax,n)
	love.graphics.setColor(100,100,100)
	love.graphics.setLineWidth(0.002*(xmax-xmin))
	local dt=(tmax-tmin)/n
	local x1,y1=curve(tmin)
	local x2,y2
	for t=tmin,tmax,dt do
		x2,y2=curve(t)
		love.graphics.line(x1,y1,x2,y2)
		x1,y1=x2,y2
	end
	x2,y2=curve(tmax)
	love.graphics.line(x1,y1,x2,y2)
end

local function point(t)
	local x,y=curve(t)
	if plotcolors then
		if y>0 then
			love.graphics.setColor(255,0,0)
		elseif y<0 then
			love.graphics.setColor(0,0,255)
		else
			love.graphics.setColor(0,255,0)
		end
	else
		love.graphics.setColor(100,100,100)
	end
	love.graphics.points(x,y)
	love.graphics.setColor(200,200,200)
	love.graphics.points(x,Ymin)
end

local function points(tmin,tmax,n)
	local dt=(tmax-tmin)/n
	for t=tmin,tmax,dt do
		point(t)
	end
end

local function randompoints(tmin,tmax,n)
	math.randomseed(seed)
	for i=1,n do
		local t=random(tmin,tmax)
		point(t)
	end
end

local function plot(xmin,xmax,ymin,ymax,tmin,tmax,n)
	coords(xmin,xmax,ymin,ymax)
	axes(xmin,xmax,ymin,ymax)
	if plotgraph then
		graph(xmin,xmax,ymin,ymax,tmin,tmax,500)
	end
	if plotpoints then
		if plotuniform then
			points(tmin,tmax,n)
		else
			randompoints(tmin,tmax,n)
		end
	end
end

function love.load()
	--love.window.setFullscreen(true)
	love.window.setMode(640,480,{resizable=true})
	love.window.setPosition(0,0)
	love.window.setTitle("curve sampling")
	love.graphics.setBackgroundColor(255,255,255)
	love.graphics.clear()
	love.graphics.setFont(love.graphics.newFont(96))
	love.keyboard.setKeyRepeat(true)
	love.graphics.setPointSize(8)
end

local N=0

function love.keypressed(key)
	if key == 'q' or key == 'escape' then
		love.event.quit()
	elseif key == 's' then
		love.graphics.newScreenshot():encode('png','output.png')
	elseif key == '[' then
		N=N-1
		if N<0 then N=0 end
	elseif key == ']' then
		N=N+1
		if N>15 then N=15 end
	elseif key == 'p' then
		plotpoints= not plotpoints
	elseif key == 'g' then
		plotgraph= not plotgraph
	elseif key == 'u' then
		plotuniform= not plotuniform
	elseif key == 'c' then
		plotcolors= not plotcolors
	elseif key == 'r' then
		plotuniform= false
		seed=seed+1
	end
end

function love.draw()
	plot(-0,6,Ymin,1,0,2*math.pi,2^N)
end
