local plotborders=false
local usecorners=false

local function f(x,y)
	--local v=y-x^2
	--local v=(y-x^2)*((x-1.123)^2+y^2-0.003)
	-- local v=y^2-x^3+x+0.38490017945975
	local v=y^2-x^3+x
	if v>0 then
		return "inside"
	elseif v<0 then
		return "outside"
	else
		return "border"
	end
end

local function coords(xmin,xmax,ymin,ymax)
	local W,H=love.graphics.getDimensions()
	love.graphics.translate(0,H)
	love.graphics.scale(1,-1)
	local dx=xmax-xmin
	local dy=ymax-ymin
	local s=math.min(W/dx,H/dy)
	love.graphics.translate((W-s*dx)/2,(H-s*dy)/2)
	love.graphics.scale(s,s)
	love.graphics.translate(-xmin,-ymin)
	love.graphics.setLineWidth(0)
end

local function box(xmin,xmax,ymin,ymax,r,g,b)
	love.graphics.setColor(r,g,b)
	love.graphics.rectangle("fill",xmin,ymin,xmax-xmin,ymax-ymin)
	if not plotborders then
		love.graphics.setColor(200,200,200)
		love.graphics.rectangle("line",xmin,ymin,xmax-xmin,ymax-ymin)
	end
end

local function classify(xmin,xmax,ymin,ymax)
	if not usecorners then
		local xmid=(xmin+xmax)/2
		local ymid=(ymin+ymax)/2
		return f(xmid,ymid)
	else
		local f1=f(xmin,ymin)
		local f2=f(xmax,ymin)
		local f3=f(xmin,ymax)
		local f4=f(xmax,ymax)
		if f1==f2 and f1==f3 and f1==f4 then
			return f1
		else
			return "border"
		end
	end
end

local function explore(xmin,xmax,ymin,ymax,level)
	if level==0 then
		local c=classify(xmin,xmax,ymin,ymax)
		if c=="inside" then
			box(xmin,xmax,ymin,ymax,255,0,0)
		elseif c=="outside" then
			box(xmin,xmax,ymin,ymax,0,0,255)
		else
			box(xmin,xmax,ymin,ymax,0,255,0)
		end
	else
		level=level-1
		local xmid=(xmin+xmax)/2
		local ymid=(ymin+ymax)/2
		explore(xmin,xmid,ymin,ymid,level)
		explore(xmin,xmid,ymid,ymax,level)
		explore(xmid,xmax,ymin,ymid,level)
		explore(xmid,xmax,ymid,ymax,level)
	end
end

local function plot(xmin,xmax,ymin,ymax,level)
	coords(xmin,xmax,ymin,ymax)
	explore(xmin,xmax,ymin,ymax,level)
	love.graphics.origin()
	love.graphics.setColor(100,100,100)
	love.graphics.print(level,10,10)
end

function love.load()
	--love.window.setFullscreen(true)
	love.window.setMode(640,480,{resizable=true})
	love.window.setPosition(0,0)
	love.window.setTitle("implicit curve plotting via sampling")
	love.graphics.setBackgroundColor(255,255,255)
	love.graphics.clear()
	love.graphics.setFont(love.graphics.newFont(96))
	love.keyboard.setKeyRepeat(true)
end

local N=0

function love.keypressed(key)
	if key == 'q' or key == 'escape' then
		love.event.quit()
	elseif key == 's' then
		love.graphics.newScreenshot():encode('png','output.png')
	elseif key == '[' then
		N=N-1
		if N<0 then N=0 end
	elseif key == ']' then
		N=N+1
		if N>10 then N=10 end
	elseif key == 'b' then
		plotborders= not plotborders
	elseif key == 'c' then
		usecorners= not usecorners
	end
end

function love.draw()
	plot(-2,3,-1,2,N)
end
