local ia=require"ia"
local interval=ia.interval
local range=ia.range
local L=0
--local L=0.384900179459750
local prune=false
local plotborders=true

local function f(x,y)
	--return y-x^2
	--return y^2-x^3+x
	--return (y-x^2)*((x-1)^2+y^2-0.01)
	--return (x^2-2)^2+(y^2-2)^2-3.5
	--return y^2-x^3+x+2/3/math.sqrt(3)
	return y^2-x^3+x+L
end

local function coords(xmin,xmax,ymin,ymax)
	local W,H=love.graphics.getDimensions()
	love.graphics.translate(0,H)
	love.graphics.scale(1,-1)
	local dx=xmax-xmin
	local dy=ymax-ymin
	local s=math.min(W/dx,H/dy)
	love.graphics.translate((W-s*dx)/2,(H-s*dy)/2)
	love.graphics.scale(s,s)
	love.graphics.translate(-xmin,-ymin)
	love.graphics.setLineWidth(0)
end

local function box(xmin,xmax,ymin,ymax,r,g,b)
	local s=0.4*255
	love.graphics.setColor(s+r,s+g,s+b)
	love.graphics.setColor(r,g,b)
	love.graphics.rectangle("fill",xmin,ymin,xmax-xmin,ymax-ymin)
	if plotborders then
		love.graphics.setColor(80,80,80)
		love.graphics.rectangle("line",xmin,ymin,xmax-xmin,ymax-ymin)
	end
end

local function classify(xmin,xmax,ymin,ymax)
	local x=interval(xmin,xmax)
	local y=interval(ymin,ymax)
	local z=f(x,y)
	local zmin,zmax=range(z)
	if zmin>0 then
		return "inside"
	elseif zmax<0 then
		return "outside"
	else
		return "border"
	end
end

local function explore(xmin,xmax,ymin,ymax,level)
	local c=classify(xmin,xmax,ymin,ymax)
	if c=="inside" then
		box(xmin,xmax,ymin,ymax,255,0,0)
		return 1
	elseif c=="outside" then
		box(xmin,xmax,ymin,ymax,0,0,255)
		return -1
	elseif level==0 then
		box(xmin,xmax,ymin,ymax,0,255,0)
		--box(xmin,xmax,ymin,ymax,0,0,0)
		return 0
	else
		level=level-1
		local xmid=(xmin+xmax)/2
		local ymid=(ymin+ymax)/2
--[[
		local dx=xmax-xmin
		local dy=ymax-ymin
		if dx>dy then
			explore(xmin,xmid,ymin,ymax,level)
			explore(xmid,xmax,ymin,ymax,level)
		else
			explore(xmin,xmax,ymin,ymid,level)
			explore(xmin,xmax,ymid,ymax,level)
		end
		do return end
--]]
		local r=
		explore(xmin,xmid,ymin,ymid,level)+
		explore(xmin,xmid,ymid,ymax,level)+
		explore(xmid,xmax,ymin,ymid,level)+
		explore(xmid,xmax,ymid,ymax,level)
		r=r/4
		if not prune then r=0 end
		if r==1 then
			box(xmin,xmax,ymin,ymax,255,0,0)
		elseif r==-1 then
			box(xmin,xmax,ymin,ymax,0,0,255)
		end
		return r
	end
end

--[[
local function box(xmin,xmax,ymin,ymax,r,g,b)
	love.graphics.setColor(r,g,b)
	love.graphics.rectangle("fill",xmin,ymin,xmax-xmin,ymax-ymin)
end

local function explore(xmin,xmax,ymin,ymax,level)
	local c=classify(xmin,xmax,ymin,ymax)
	if c=="inside" then
		if level>=0 then box(xmin,xmax,ymin,ymax,0,0,0) end
		return 1
	elseif c=="outside" then
		if level>=0 then box(xmin,xmax,ymin,ymax,255,255,255) end
		return 0
	elseif level<=0 then
		return 0.5
	else
		level=level-1
		local xmid=(xmin+xmax)/2
		local ymid=(ymin+ymax)/2
		local r=
		explore(xmin,xmid,ymin,ymid,level)+
		explore(xmin,xmid,ymid,ymax,level)+
		explore(xmid,xmax,ymin,ymid,level)+
		explore(xmid,xmax,ymid,ymax,level)
		r=r/4
		if level==0 then
			--r=0.5
			r=(1-r)*255
			box(xmin,xmax,ymin,ymax,r,r,r)
		end
		return r
	end
end
--]]

local function plot(xmin,xmax,ymin,ymax,level)
	coords(xmin,xmax,ymin,ymax)
	explore(xmin,xmax,ymin,ymax,level)
	love.graphics.origin()
	love.graphics.setColor(100,100,100)
	love.graphics.print(string.format("%d\t%.2f",level,L),10,10)
end

function love.load()
	--love.window.setFullscreen(true)
	love.window.setMode(640,480,{resizable=true})
	love.window.setPosition(0,0)
	love.window.setTitle("implicit curve plotting with interval arithmetic")
	love.graphics.setBackgroundColor(255,255,255)
	love.graphics.clear()
	love.graphics.setFont(love.graphics.newFont(96))
	love.keyboard.setKeyRepeat(true)
end

local N=0

function love.keypressed(key)
	if key == 'q' or key == 'escape' then
		love.event.quit()
	elseif key == 's' then
		love.graphics.newScreenshot():encode('png','output.png')
	elseif key == '[' then
		N=N-1
		if N<0 then N=0 end
	elseif key == ']' then
		N=N+1
		if N>=14 then N=14 end
	elseif key == 'down' then
		L=L-0.01
	elseif key == 'up' then
		L=L+0.01
	elseif key == 'b' then
		plotborders= not plotborders
	elseif key == 'p' then
		prune= not prune
	end
end

function love.draw()
	plot(-2,3,-1,2,N)
end
